@@ -1,16 +1,25 @@
 import mimetypes
 import os
 import re
+
+from io import BytesIO
+from urllib.parse import quote, unquote
+
 import requests
+
 from libthumbor import CryptoURL
 from requests.packages.urllib3.exceptions import LocationParseError
-from StringIO import StringIO
 from django.conf import settings
 from django.core.files.images import ImageFile
 from django.core.files.storage import Storage, FileSystemStorage
+from django.utils.deconstruct import deconstructible
 from . import exceptions
 
 
+# Match 'key', 'key/filename.ext' and 'key.ext'.
+THUMBOR_PATH_PATTERN = r"^/image/(?P<key>\w{32})(?:(/|\.).*){0,1}$"
+
+
 class ThumborStorageFile(ImageFile):
     def __init__(self, name, mode):
         self.name = name
@@ -23,30 +32,37 @@ class ThumborStorageFile(ImageFile):
         image_content = content.file.read()
         content.file.seek(0)
 
-        url = "%s/image" % settings.THUMBOR_RW_SERVER
+        url = f"{settings.THUMBOR_RW_SERVER}/image"
         headers = {
             "Content-Type": mimetypes.guess_type(self.name)[0] or "image/jpeg",
-            "Slug": self.name,
+            "Slug": quote(
+                self.name.encode('utf-8'), ':/?#[]@!$&\'()*+,;='),
         }
         response = requests.post(url, data=image_content, headers=headers)
-        self._location = response.headers["location"]
-        return super(ThumborStorageFile, self).write(image_content)
+        if response.status_code != 201:
+            raise exceptions.ThumborPostException(response)
+        self._location = unquote(response.headers["location"])
+        try:
+            self._location = self._location.decode('utf-8')
+        except AttributeError:
+            pass
+        return super().write(image_content)
 
     def delete(self):
-        url = "%s%s" % (settings.THUMBOR_RW_SERVER, self.name)
+        url = f"{settings.THUMBOR_RW_SERVER}{self.name}"
         response = requests.delete(url)
         if response.status_code == 405:
             raise exceptions.MethodNotAllowedException
-        elif response.status_code == 404:
+        if response.status_code == 404:
             raise exceptions.NotFoundException
-        elif response.status_code == 204:
+        if response.status_code == 204:
             return
 
     def _get_file(self):
-        if self._file is None:
-            self._file = StringIO()
+        if self._file is None or self._file.closed:
+            self._file = BytesIO()
             if 'r' in self._mode:
-                url = "%s%s" % (settings.THUMBOR_RW_SERVER, self.name)
+                url = f"{settings.THUMBOR_RW_SERVER}{self.name}"
                 response = requests.get(url)
                 self._file.write(response.content)
                 self._file.seek(0)
@@ -62,7 +78,12 @@ class ThumborStorageFile(ImageFile):
         self.seek(0, os.SEEK_END)
         return self.tell()
 
+    def close(self):
+        super().close()
+        self._file = None
 
+
+@deconstructible
 class ThumborStorage(Storage):
     """Thumbor Simple Storage Service"""
 
@@ -88,11 +109,10 @@ class ThumborStorage(Storage):
 
     def exists(self, name):
         # name is the location returned by Thumbor when posted > may exists.
-        if re.match(r"^/image/\w{32}(?:/.*){0,1}$", name):
+        if re.match(THUMBOR_PATH_PATTERN, name):
             return thumbor_original_exists(thumbor_original_image_url(name))
         # name as defined in 'upload_to' > new image.
-        else:
-            return False
+        return False
 
     def size(self, name):
         f = self.open(name)
@@ -102,18 +122,19 @@ class ThumborStorage(Storage):
         return thumbor_image_url(self.key(name))
 
     def key(self, name):
-        return re.match(r"^/image/(?P<key>\w{32})(?:/.*){0,1}$", name).groupdict()['key']
+        return re.match(THUMBOR_PATH_PATTERN, name).groupdict()['key']
 
-    def get_available_name(self, name):
+    def get_available_name(self, name, max_length=None):
         # There is no way to know if the image exists on Thumbor.
         # When posting a new original image, Thumbor generate a ramdom unique id as key.
         # http://en.wikipedia.org/wiki/Universally_unique_identifier
         # https://github.com/globocom/thumbor/blob/ae9a150e8a2b771dd49b4137186e9fdfbea09733/thumbor/handlers/images.py#L51
         return name
 
-    #TODO : get_valid_name(name)
+    # TODO : get_valid_name(name)
 
 
+@deconstructible
 class ThumborMigrationStorage(ThumborStorage, FileSystemStorage):
     """A Storage that fallback on the FileSystemStorage when retrieving the image.
 
@@ -140,41 +161,35 @@ class ThumborMigrationStorage(ThumborStorage, FileSystemStorage):
     def _open(self, name, mode='rb'):
         if self.is_thumbor(name):
             return ThumborStorage._open(self, name, mode)
-        else:
-            return FileSystemStorage._open(self, name, mode)
+        return FileSystemStorage._open(self, name, mode)
 
     def delete(self, name):
         if self.is_thumbor(name):
             return ThumborStorage.delete(self, name)
-        else:
-            return FileSystemStorage.delete(self, name)
+        return FileSystemStorage.delete(self, name)
 
     def exists(self, name):
         if self.is_thumbor(name):
             return ThumborStorage.exists(self, name)
-        else:
-            return FileSystemStorage.exists(self, name)
+        return FileSystemStorage.exists(self, name)
 
     def url(self, name):
         if self.is_thumbor(name):
             return ThumborStorage.url(self, name)
-        else:
-            return FileSystemStorage.url(self, name)
+        return FileSystemStorage.url(self, name)
 
     def key(self, name):
         if self.is_thumbor(name):
             return ThumborStorage.key(self, name)
-        else:
-            raise NotImplementedError
+        raise NotImplementedError
 
     def path(self, name):
         if self.is_thumbor(name):
             return ThumborStorage.path(self, name)
-        else:
-            return FileSystemStorage.path(self, name)
+        return FileSystemStorage.path(self, name)
 
     def is_thumbor(self, name):
-        return re.match(r"^/image/\w{32}(?:/.*){0,1}$", name)
+        return re.match(THUMBOR_PATH_PATTERN, name)
 
 
 def thumbor_original_exists(url):
@@ -188,27 +203,26 @@ def thumbor_original_exists(url):
         return False
     if response.status_code == 200:
         return True
-    else:
-        return False
+    return False
 
 
-## These functions because some methods in ThumborStorage may be called with
-#   self being a ThumborMigrationStorage instance and result in infinite loop.
+# These functions because some methods in ThumborStorage may be called with
+# self being a ThumborMigrationStorage instance and result in infinite loop.
 # These methods proxiing to these functions.
 
 def thumbor_image_url(key):
     crypto = CryptoURL(key=settings.THUMBOR_SECURITY_KEY)
-    return "%s%s" % (settings.THUMBOR_SERVER, crypto.generate(image_url=key))
+    return f"{settings.THUMBOR_SERVER}{crypto.generate(image_url=key)}"
 
 
 def thumbor_original_image_url(name):
-    return "%s%s" % (settings.THUMBOR_RW_SERVER, name)
+    return f"{settings.THUMBOR_RW_SERVER}{name}"
 
 
-## Utils
+# Utils
 
 def readonly_to_rw_url(readonly_url):
-    key = re.match(r"^%s/(?P<secu>[\w\-=]{28})/(?P<key>\w{32})(?:/.*){0,1}$" %
-             re.escape(settings.THUMBOR_SERVER), readonly_url).groupdict()["key"]
-    name = "/image/%s" % key
+    matches = re.match(r"^%s/(?P<secu>[\w\-=]{28})/(?P<key>\w{32})(?P<extra>(?:.*))$" %
+                       re.escape(settings.THUMBOR_SERVER), readonly_url).groupdict()
+    name = f"/image/{matches['key']}{matches['extra']}"
     return thumbor_original_image_url(name)
